@@ -28,7 +28,8 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 
 import dxpy
 from . import DXDataObject, DXJob
-from ..utils import merge, instance_type_to_sys_reqs
+from ..utils import merge
+from ..system_requirements import SystemRequirementsDict
 from ..exceptions import DXError
 from ..compat import basestring
 
@@ -46,7 +47,7 @@ class DXExecutable:
         '''
         Takes the same arguments as the run method. Creates an input hash for the /executable-xxxx/run method,
         translating ONLY the fields that can be handled uniformly across all executables: project, folder, name, tags,
-        properties, details, depends_on, allow_ssh, debug, delay_workspace_destruction, and extra_args.
+        properties, details, depends_on, allow_ssh, debug, delay_workspace_destruction, ignore_reuse, and extra_args.
         '''
         project = kwargs.get('project') or dxpy.WORKSPACE_ID
 
@@ -55,8 +56,10 @@ class DXExecutable:
             if kwargs.get(arg) is not None:
                 run_input[arg] = kwargs[arg]
 
-        if kwargs.get('instance_type') is not None:
-            run_input["systemRequirements"] = instance_type_to_sys_reqs(kwargs['instance_type'])
+        if kwargs.get('instance_type') is not None or kwargs.get('cluster_spec') is not None:
+            instance_type_srd = SystemRequirementsDict.from_instance_type(kwargs.get('instance_type'))
+            cluster_spec_srd = SystemRequirementsDict(kwargs.get('cluster_spec'))
+            run_input["systemRequirements"] = (instance_type_srd + cluster_spec_srd).as_dict()
 
         if kwargs.get('depends_on') is not None:
             run_input["dependsOn"] = []
@@ -85,6 +88,9 @@ class DXExecutable:
         if kwargs.get('priority') is not None:
             run_input["priority"] = kwargs['priority']
 
+        if kwargs.get('ignore_reuse') is not None:
+            run_input["ignoreReuse"] = kwargs['ignore_reuse']
+
         if dxpy.JOB_ID is None:
             run_input["project"] = project
 
@@ -101,9 +107,10 @@ class DXExecutable:
         '''
         # Although it says "for_applet", this is factored out of
         # DXApplet because apps currently use the same mechanism
-        for unsupported_arg in ['stage_instance_types', 'stage_folders', 'rerun_stages']:
+        for unsupported_arg in ['stage_instance_types', 'stage_folders', 'rerun_stages', 'ignore_reuse_stages']:
             if kwargs.get(unsupported_arg):
                 raise DXError(unsupported_arg + ' is not supported for applets (only workflows)')
+
         return DXExecutable._get_run_input_common_fields(executable_input, **kwargs)
 
     def _run_impl(self, run_input, **kwargs):
@@ -149,9 +156,9 @@ class DXExecutable:
         raise NotImplementedError('_get_cleanup_keys is not implemented')
 
     def run(self, executable_input, project=None, folder=None, name=None, tags=None, properties=None, details=None,
-            instance_type=None, stage_instance_types=None, stage_folders=None, rerun_stages=None,
+            instance_type=None, stage_instance_types=None, stage_folders=None, rerun_stages=None, cluster_spec=None,
             depends_on=None, allow_ssh=None, debug=None, delay_workspace_destruction=None, priority=None,
-            extra_args=None, **kwargs):
+            ignore_reuse=None, ignore_reuse_stages=None, extra_args=None, **kwargs):
         '''
         :param executable_input: Hash of the executable's input arguments
         :type executable_input: dict
@@ -179,6 +186,10 @@ class DXExecutable:
         :type delay_workspace_destruction: boolean
         :param priority: Priority level to request for all jobs created in the execution tree, either "normal" or "high"
         :type priority: string
+        :param ignore_reuse: Disable job reuse for this execution
+        :type ignore_reuse: boolean
+        :param ignore_reuse_stages: Stages of a workflow (IDs, names, or indices) or "*" for which job reuse should be disabled
+        :type ignore_reuse_stages: list
         :param extra_args: If provided, a hash of options that will be merged into the underlying JSON given for the API call
         :type extra_args: dict
         :returns: Object handler of the newly created job
@@ -203,8 +214,11 @@ class DXExecutable:
                                         stage_instance_types=stage_instance_types,
                                         stage_folders=stage_folders,
                                         rerun_stages=rerun_stages,
+                                        cluster_spec=cluster_spec,
                                         depends_on=depends_on,
                                         allow_ssh=allow_ssh,
+                                        ignore_reuse=ignore_reuse,
+                                        ignore_reuse_stages=ignore_reuse_stages,
                                         debug=debug,
                                         delay_workspace_destruction=delay_workspace_destruction,
                                         priority=priority,
@@ -217,9 +231,9 @@ class DXExecutable:
 # DXApplet #
 ############
 _applet_required_keys = ['name', 'title', 'summary', 'types', 'tags',
-                         'properties', 'dxapi', 'inputSpec', 'outputSpec',
+                         'httpsApp', 'properties', 'dxapi', 'inputSpec', 'outputSpec',
                          'runSpec', 'access', 'details']
-_applet_optional_keys = []
+_applet_optional_keys = ['ignoreReuse']
 _applet_describe_output_keys = ['properties', 'details']
 _applet_cleanup_keys = ['name', 'title', 'summary', 'types', 'tags',
                         'properties', 'runSpec', 'access', 'details']
@@ -237,7 +251,7 @@ class DXApplet(DXDataObject, DXExecutable):
 
        The applet's run specification (a dict indicating, among other things, how the code of the
        applet is to be interpreted). See `the API docs for Run Specification
-       <https://wiki.dnanexus.com/API-Specification-v1.0.0/IO-and-Run-Specifications#Run-Specification>`_
+       <https://documentation.dnanexus.com/developer/api/running-analyses/io-and-run-specifications#run-specification>`_
        for more information.
 
     .. py:attribute:: dxapi
@@ -249,7 +263,7 @@ class DXApplet(DXDataObject, DXExecutable):
        The applet's access requirements hash (a dict indicating any nonstandard permissions, such
        as requiring access to the internet, that are needed by the applet). See `the API docs for
        Access Requirements
-       <https://wiki.dnanexus.com/API-Specification-v1.0.0/IO-and-Run-Specifications#Access-Requirements>`_
+       <https://documentation.dnanexus.com/developer/api/running-analyses/io-and-run-specifications#access-requirements>`_
        for more information.
 
     .. py:attribute:: title
@@ -313,12 +327,12 @@ class DXApplet(DXDataObject, DXExecutable):
 
         .. note:: It is highly recommended that the higher-level module
            :mod:`dxpy.app_builder` or (preferably) its frontend `dx build
-           <https://wiki.dnanexus.com/Command-Line-Client/Index-of-dx-Commands#build>`_
+           <https://documentation.dnanexus.com/user/helpstrings-of-sdk-command-line-utilities#build>`_
            be used instead for applet creation.
 
         Creates an applet with the given parameters. See the API
         documentation for the `/applet/new
-        <https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets-and-Entry-Points#API-method:-/applet/new>`_
+        <https://documentation.dnanexus.com/developer/api/running-analyses/applets-and-entry-points#api-method-applet-new>`_
         method for more info. The applet is not run until :meth:`run()`
         is called.
 
@@ -344,7 +358,7 @@ class DXApplet(DXDataObject, DXExecutable):
         Returns the contents of the applet. The result includes the
         key-value pairs as specified in the API documentation for the
         `/applet-xxxx/get
-        <https://wiki.dnanexus.com/API-Specification-v1.0.0/Applets-and-Entry-Points#API-method:-/applet-xxxx/get>`_
+        <https://documentation.dnanexus.com/developer/api/running-analyses/applets-and-entry-points#api-method-applet-xxxx-get>`_
         method.
         """
         return dxpy.api.applet_get(self._dxid, **kwargs)
